#ifndef __DUNE_GRID_HOWTO_FUNCTORS_HH__
#define __DUNE_GRID_HOWTO_FUNCTORS_HH__

#include <dune/common/fvector.hh>

// Skeleton of Functor for evaluation of PK simplex function
template<typename LocalFunction>
class LocalFunctor{
private:
  typedef typename LocalFunction::Traits::LocalBasisType::Traits::RangeType RT;
  typedef typename LocalFunction::Traits::LocalBasisType::Traits::RangeFieldType RFT;
  typedef typename LocalFunction::Traits::LocalBasisType::Traits::DomainType DT;
  typedef std::vector<RFT> Coefficients;

  const LocalFunction local_function;
  const Coefficients coeffs;

public:
  LocalFunctor(const LocalFunction & l_, const Coefficients & coeffs_) 
    : local_function(l_), coeffs(coeffs_)
  {}

  RT operator() (const DT & x) const
  {

    //!! Modify code here
    return 1;
  }
};

template< typename F1, typename F2>
class ProductFunctor
{
  const F1 & f1;
  const F2 & f2;
public:
  ProductFunctor(const F1 & f1_, const F2 & f2_)
    : f1(f1_), f2(f2_)
  {}

  template<typename Coordinate>
  double operator() (const Coordinate& x) const
  {
    return f1(x)*f2(x);
  }
};


/**
   This function may be used by the VTKWriter object to evaluate the
   functor object at a given local coordinate.
 */
template<class Grid, class Functor>
class FunctorVTKFunction : public Dune::VTKFunction<Grid>
{
private:

  // Some types which are exported by the base class
  typedef Dune::VTKFunction<Grid> Base;
  typedef typename Base::Entity Entity;
  typedef typename Base::ctype ctype;
  enum { dim =  Base::dim };

  // Constant reference to the functor object.
  const Functor & f;
  const std::string data_name;
public:

  FunctorVTKFunction(const Functor & f_, std::string name_) : f(f_), data_name(name_)
  {}

  //! The number of components of this function. This is necessary as
  //! the VTKWriter would also allow the visualization of vector
  //! valued functions.
  int ncomps() const {return 1;}

  //! Evaluate the functor at a given local coordinate
  double evaluate(int comp, const Entity & e, const Dune::FieldVector<ctype,dim> & xi) const
  {
    typedef typename Entity::Geometry::GlobalCoordinate GlobalCoordinate;
    const GlobalCoordinate c = e.geometry().global(xi);
    const double f_value = f(c);
    return f_value;
  }

  //! This name will later be displayed as a label for the visualized
  //! data in paraview.
  std::string name() const
  {
    return data_name;
  }
};


#endif // __DUNE_GRID_HOWTO_FUNCTORS_HH__
